/*
 * Copyright 2024 Luca Fulchir <luca.fulchir@runesauth.com>
 *
 * Licensed under the Apache License, Version 2.0 with LLVM exception (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License and of the exception at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     https://spdx.org/licenses/LLVM-exception.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

use ::std::any::Any;

// Package stuff

/// Get a package from its module and export it in the current module
#[macro_export]
macro_rules! moduse {
    ($($name:ident,)*) => {
        $(
            mod $name;
            #[allow(unused_imports)]
            use $name::*;
        )*
    };
}
// re-export `paste` crate for next macros
pub use paste;

/// Add multipla packages to a repo
/// e.g.:
/// bok::repo_packages! {
///   MyRepo {
///     Mypkg1,
///     Mypkg2,
///   }
/// }
#[macro_export]
macro_rules! repo_packages {
    ( $repo:ident { $($name:ident,)*} )  => {
        impl $repo {
            $crate::packages!{$($name ,)*}
        }
    };
}

/// Add multipla packages to a repo Impl
/// e.g.:
/// impl MyRepo {
///   bok::packages! {
///     Mypkg1,
///     Mypkg2,
///   }
/// }
#[macro_export]
macro_rules! packages {
    ($($name:ident,)*) => {
        $crate::paste::paste! {
            $(
                pub fn [<$name:snake>] (&self) -> $name {
                    $name::default()
                }
            )*
        }
    };
}

pub trait Repository: Default {}

///
/// This is an empty Package List. Will be available in the main library
///
#[derive(::std::default::Default)]
pub struct RepositoryEmpty {}
impl Repository for RepositoryEmpty {}

pub trait Pkg: ::std::default::Default + Any {
    fn as_any(&self) -> &dyn Any
    where
        Self: Sized,
    {
        self
    }
}

// Conf stuff
pub struct Conf {}

/*
pub enum Value<T: Sized> {
    /// Set, only once per
    Set(T),
    /// Delete a value
    Del(T),
}
pub enum ValueList<T: Sized> {
    /// Set, only once per
    Set(T),
    /// Delete a value
    Del(T),
    /// Add to a list
    Add(T),
    /// Merge (add + dedup)
    Merge(T),
}
*/
